# Copyright (c) 2020 kien@clare.ai.
# All Rights Reserved.

# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at

#   http://www.apache.org/licenses/LICENSE-2.0

# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.

import copy
import json
import logging
import socket

import requests
import simplejson

from wati_api import exceptions

USER_AGENT = 'wati_api'
LOG = logging.getLogger(__name__)


class HTTPClient(object):
    def __init__(self, endpoint, **kwargs):
        self.endpoint = endpoint.strip('/')
        if not endpoint.startswith('http') and not endpoint.startswith('https'):
            self.endpoint = 'http://{}'.format(endpoint)
        self.timeout = float(kwargs.get('timeout', 600))
        self.session = requests.Session()
        self.session.headers["User-Agent"] = USER_AGENT

    def __del__(self):
        if self.session:
            try:
                self.session.close()
            except Exception as e:
                LOG.exception(e)
            finally:
                self.session = None

    def _request(self, method, url, body=None, **kwargs):
        """Send an http request with the specified characteristics.
        """
        # Copy the kwargs so we can reuse the original in case of redirects
        headers = copy.deepcopy(kwargs.pop('headers', {}))
        if headers.get('Content-Type', 'application/json') is None:
            headers['Content-Type'] = 'application/json'
        if self.endpoint.endswith("/") or url.startswith("/"):
            conn_url = "%s%s" % (self.endpoint, url)
        else:
            conn_url = "%s/%s" % (self.endpoint, url)
        try:
            resp = self.session.request(method, conn_url,
                                        json=body, headers=headers,
                                        timeout=self.timeout, **kwargs)
        except requests.exceptions.Timeout as e:
            message = ("Error communicating with %(url)s: %(e)s" %
                       dict(url=conn_url, e=e))
            raise exceptions.InvalidEndpoint(message=message)
        except requests.exceptions.ConnectionError as e:
            message = ("Error finding address for %(url)s: %(e)s" %
                       dict(url=conn_url, e=e))
            raise exceptions.CommunicationError(message=message)
        except socket.gaierror as e:
            message = "Error finding address for %s: %s" % (
                self.endpoint, e)
            raise exceptions.InvalidEndpoint(message=message)
        except (socket.error, socket.timeout, IOError) as e:
            endpoint = self.endpoint
            message = ("Error communicating with %(endpoint)s %(e)s" %
                       {'endpoint': endpoint, 'e': e})
            raise exceptions.CommunicationError(message=message)
        except Exception as e:
            raise e

        LOG.debug('%(method)s call to image for %(url)s.',
                  {'method': resp.request.method,
                   'url': resp.url})
        return self._handle_response(resp)

    def _handle_response(self, response):
        try:
            response.raise_for_status()
        except requests.exceptions.HTTPError as e:
            err_msg = str(e)

            # Attempt to get Error message from response
            try:
                error_dict = response.json()
            except (json.decoder.JSONDecodeError,
                    simplejson.errors.JSONDecodeError):
                pass
            else:
                err_msg += " [Error: {}]".format(error_dict)
            raise requests.exceptions.HTTPError(err_msg)
        else:
            return response

    def head(self, url, **kwargs):
        return self._request('HEAD', url, **kwargs)

    def get(self, url, **kwargs):
        return self._request('GET', url, **kwargs)

    def post(self, url, **kwargs):
        return self._request('POST', url, **kwargs)

    def put(self, url, **kwargs):
        return self._request('PUT', url, **kwargs)

    def patch(self, url, **kwargs):
        return self._request('PATCH', url, **kwargs)

    def delete(self, url, **kwargs):
        return self._request('DELETE', url, **kwargs)



# Copyright (c) 2020 kien@clare.ai.
# All Rights Reserved.

# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at

#   http://www.apache.org/licenses/LICENSE-2.0

# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.

import json
import logging
import time

import requests

from wati_api import http
from wati_api import utils

LOG = logging.getLogger(__name__)


class Client(http.HTTPClient):
    """Client for the WATI API.
    :param endpoint: A user-supplied endpoint URL for the WATI service.
    :param token: A token get from: https://app.wati.io/register
    """

    def __init__(self, endpoint, token, **kwargs):
        """Initialize a new client for the WATI API."""
        super(Client, self).__init__(endpoint, **kwargs)
        self.headers = {
            'Authorization': f'Bearer {token}',
            'Content-Type': 'application/json'
        }

    def get_messages(self, whatsapp_number, page_size=None, page_number=None):
        """Get Messages by whatsapp number

        :param whatsapp_number(str): Whatsapp Number
        :param page_size(int): Page size
        :param page_number(int): Page number
        """
        url = utils.generate_url('/api/v1/getMessages/', whatsapp_number,
                                 pageSize=page_size,
                                 pageNumber=page_number)
        return self.get(url, headers=self.headers).json()

    def get_message_templates(self, page_size=None, page_number=None):
        """Get Message Templates

        :param page_size(int): Page size
        :param page_number(int): Page number
        """
        url = utils.generate_url('/api/v1/getMessageTemplates',
                                 pageSize=page_size,
                                 pageNumber=page_number)
        return self.get(url, headers=self.headers).json()

    def get_contacts(self, page_size=None, page_number=None,
                     name=None, attribute=None, createdDate=None):
        """Get Contacts List

        :param page_size(int): Page size
        :param page_number(int): Page number
        :param name(str): Contact name
        :param attribute(string): Attribute (parameters format:
                [{name: "name", operator: "contain", value: "test"}] )
        :poram createdData(str): Created Date (YYYY-MM-DD or MM-DD-YYYY)
        """
        url = utils.generate_url('/api/v1/getContacts',
                                 pageSize=page_size,
                                 pageNumber=page_number,
                                 name=name, attribute=attribute,
                                 createdDate=createdDate)
        return self.get(url, headers=self.headers).json()

    def get_media(self, file_name):
        """Get media by file name

        param file_name(str): File name
        """
        url = utils.generate_url('/api/v1/getMedia', fileName=file_name)
        return self.get(url, headers=self.headers, stream=True).raw

    def update_contact_attributes(self, whatsapp_number, custom_params):
        """Update Contact Attributes

        :param whatsapp_number(str): Whatsapp number
        :param custom_params(list):
                [
                    {
                    "name": "string",
                    "value": "string"
                    }
                ]
        """
        payload = {
            "customParams": custom_params
        }
        url = utils.generate_url(
            '/api/v1/updateContactAttributes', whatsapp_number)
        return self.post(url, headers=self.headers,
                         body=payload).json()

    def add_contact(self, whatsapp_number, name, custom_params):
        """Add Contact

        :param whatsapp_number(str): Whatsapp number
        :param name(str):
        :param custom_params(list):
                [
                    {
                    "name": "string",
                    "value": "string"
                    }
                ]
        """
        payload = {
            "name": name,
            "customParams": custom_params
        }
        url = utils.generate_url('/api/v1/addContact', whatsapp_number)
        return self.post(url, headers=self.headers,
                         body=payload).json()

    def send_session_message(self, whatsapp_number, message_text):
        """Send Message to opened session

        :param whatsapp_number(str): Whatsapp number
        :param message_text(str): Message
        """
        url = utils.generate_url('/api/v1/sendSessionMessage', whatsapp_number,
                                 messageText=message_text)
        return self.post(url, headers=self.headers).json()

    def send_template_message_csv(self, template_name, broadcast_name,
                                  whatsapp_number_csv):
        """Send template messages (CSV)

        :param template_name(str): Defined in the Template Message under
                                   Broadcast
        :param broadcast_name(str): Broadcast name
        :param whatsapp_number_csv(str): Csv file absolute path
        """
        url = utils.generate_url('/api/v1/sendTemplateMessageCSV',
                                 template_name=template_name,
                                 broadcast_name=broadcast_name)
        headers = {
            'Authorization': self.headers['Authorization'],
            'Content-Type': 'multipart/form-data'
        }
        files = {'file': open(whatsapp_number_csv).read()}
        return self.post(url, headers=headers,
                         files=files).json()

    def send_template_message(self, whatsapp_number, broadcast_name,
                              template_name, template_parameters):
        """Send message templates"""
        url = utils.generate_url(
            '/api/v1/sendTemplateMessage/', whatsapp_number)
        payload = {
            "template_name": template_name,
            "broadcast_name": broadcast_name,
            "parameters": json.dumps(template_parameters)
        }
        return self.post(url, body=payload, headers=self.headers).json()



# Copyright (c) 2020 kien@clare.ai.
# All Rights Reserved.

# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at

#   http://www.apache.org/licenses/LICENSE-2.0

# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.


def generate_url(url, *args, **kwargs):
    """Generate an url with input arguments.
    *args -> path parameter.
    *kwargs -> query parameter.
    """
    url = url.rstrip('/')
    if args:
        url = '/'.join([url, *args])
    if kwargs:
        for k, v in kwargs.items():
            # If there is None or empty value, just by pass it
            if not v:
                continue
            if k == list(kwargs.keys())[0]:
                url += "?{}={}" . format(k, v)
                continue
            url += "&{}={}" . format(k, v)
    return url



# Copyright (c) 2020 kien@clare.ai.
# All Rights Reserved.

# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at

#   http://www.apache.org/licenses/LICENSE-2.0

# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.

import os
import json
import hashlib

from wati_api import client

if __name__ == '__main__':
    try:
        endpoint = os.environ['WATI_API_ENDPOINT']
        token = os.environ['WATI_API_TOKEN']
    except KeyError as e:
        print('Missing environment variables! %s' % str(e))
        raise e
    wa_client = client.Client(endpoint, token)
    print("# Get message API")
    print(wa_client.get_messages("919920842422"))
    print("# Get message templates API")
    print(wa_client.get_message_templates())
    print("# Get contact list API")
    print(wa_client.get_contacts())
    # Need to recheck - 404
    # print("# Get media API")
    # print(wa_client.get_media('ABC'))
    print("# Add contact API")
    custom_params = [
        {
            "name": "member",
            "value": "VIP"
        }
    ]
    print(wa_client.add_contact("919920842422", "ABC", custom_params))
    print("# Update contact attribute")
    print(wa_client.update_contact_attributes("919920842422", custom_params))
    print("# Send Session message")
    print(wa_client.send_session_message("919920842422", "Hello"))
    template_parameters = [
        {
            'name': 'name',
            'value': 'John'
        },
        {
            'name': 'ordernumber',
            'value': '12345'
        }
    ]
    print("# Send template message API")
    print(wa_client.send_template_message(
        whatsapp_number="919920842422",
        broadcast_name="order_update",
        template_name="order_update",
        template_parameters=template_parameters
    ))
